﻿-- 272 D ST BOSTON MA 02127 

SELECT osm_id, ST_DistanceSphere(X.the_geom, ST_SetSRID(ST_Point(-71.0506158,42.339387),4326)) AS dist_m
FROM osm.ways_vertices_pgr X
ORDER BY X.the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1;

SELECT ST_Point(-71.0506158,42.339387) AS geom;

SELECT X.seq, X.node, X.edge, X.cost/60.0 AS cost_in_minutes, X.agg_cost/60.0 AS agg_cost_in_minutes
FROM 
    pgr_drivingDistance(
        'SELECT gid AS id, cost_s As cost, reverse_cost_s As reverse_cost, source, target FROM osm.ways',
        (SELECT id FROM osm.ways_vertices_pgr ORDER BY the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1),
        60 * 1,
        directed := TRUE) AS X
    INNER JOIN 
    osm.ways_vertices_pgr Y ON X.node = Y.id;

-- OJ
SELECT Z.the_geom
FROM 
    pgr_drivingDistance(
        'SELECT gid AS id, cost_s As cost, reverse_cost_s As reverse_cost, source, target FROM osm.ways',
        (SELECT id FROM osm.ways_vertices_pgr ORDER BY the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1),
        60 * 1,
        directed := TRUE) AS X
    INNER JOIN 
    osm.ways_vertices_pgr Y ON X.node = Y.id
    INNER JOIN
    osm.ways Z 
    ON Z.source = Y.id OR Z.target = Y.id;

SELECT Z.the_geom
FROM 
    pgr_drivingDistance(
        'SELECT gid AS id, ABS(cost_s) As cost, reverse_cost_s As reverse_cost, source, target FROM osm.ways',
        (SELECT id FROM osm.ways_vertices_pgr ORDER BY the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1),
        60 * 2,
        directed := FALSE) AS X
    INNER JOIN 
    osm.ways_vertices_pgr Y ON X.node = Y.id
    INNER JOIN
    osm.ways Z 
    ON Z.source = Y.id OR Z.target = Y.id;


-- Nodes of catchment areas
SELECT 
    D.seq AS id, 
    V.the_geom
FROM 
    pgr_drivingDistance(
        $sql$
            SELECT gid As id, source, target, cost_s AS cost, reverse_cost_s AS reverse_cost FROM osm.ways
        $sql$,
        (SELECT id FROM osm.ways_vertices_pgr N ORDER BY N.the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1), 
        1 * 60, 
        TRUE
    ) D INNER JOIN 
    osm.ways_vertices_pgr V ON D.node = V.id
;

-- Convex hull of catchement area (OJ)
SELECT 
    1 AS id, 
    ST_ConvexHull(ST_Union(v.the_geom)) 
FROM 
    pgr_drivingDistance(
        $$
            SELECT gid As id, source, target, cost_s AS cost, reverse_cost_s AS reverse_cost FROM osm.ways
        $$,
        (SELECT id FROM osm.ways_vertices_pgr N ORDER BY N.the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1), 
        1 * 60, 
        TRUE
    ) AS D INNER JOIN 
osm.ways_vertices_pgr V ON D.node = V.id
;

-- Alphashape of catchment area (OJ)
SELECT 
    1 As id, 
    ST_SetSRID(
        pgr_pointsAsPolygon(
            $$
                SELECT D.seq AS id, ST_X(V.the_geom) AS x, ST_Y(V.the_geom) As y 
                FROM 
                    pgr_drivingDistance(
                        $sql$
                            SELECT gid As id, source, target, cost_s AS cost, reverse_cost_s AS reverse_cost FROM osm.ways
                        $sql$,
                        (SELECT id FROM osm.ways_vertices_pgr N ORDER BY N.the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1), 
                        1 * 60, 
                        TRUE
                    ) D INNER JOIN 
                    osm.ways_vertices_pgr V ON D.node = V.id
            $$
        ),
    4326
    ) As geom
;


-- OJ isochrons 1-5 minutes alphashape determine service area with points as polygon, isochrones 1-5
SELECT 
    i As id, 
    ST_SetSRID(
        pgr_pointsAsPolygon(
            'SELECT D.seq AS id, ST_X(V.the_geom) AS x, ST_Y(V.the_geom) As y
            FROM 
                pgr_drivingDistance(
                    $$
                        SELECT gid As id, source, target, cost_s AS cost, reverse_cost_s AS reverse_cost FROM osm.ways
                    $$,
                    (SELECT id FROM osm.ways_vertices_pgr AS n ORDER BY n.the_geom <-> ST_SetSRID(ST_Point(-71.0506158,42.339387),4326) LIMIT 1), 
                    60 * ' || i || ', 
                    TRUE
                ) D INNER JOIN 
                osm.ways_vertices_pgr v ON D.node = V.id'
        ), 
        4326
    ) As geom
FROM generate_series(1,5) AS i;

-- Multiples nodes driving_distance

CREATE TABLE fire_stations(id serial PRIMARY KEY, address varchar(255), geom geometry(Point,4326), closest_node integer);

INSERT INTO fire_stations(address, geom)
VALUES 
    ('700 East 4th St,Boston, MA', ST_GeomFromText('POINT(-71.036383 42.334838)', 4326)),
    ('125 Purchase St, Boston, MA', ST_GeomFromText('POINT(-71.052946 42.35479)', 4326)),
    ('272 D ST, Boston, MA', ST_SetSRID(ST_Point(-71.0506158, 42.339387), 4326))
;

UPDATE fire_stations SET closest_node = (SELECT id FROM osm.ways_vertices_pgr N ORDER BY N.the_geom <-> fire_stations.geom LIMIT 1);

SELECT X.seq, X.node, X.edge, Z.name, Y.the_geom, X.cost/60.0 AS cost_in_minutes, X.agg_cost/60.0 AS agg_cost_in_minutes
FROM 
    pgr_drivingDistance(
        'SELECT gid AS id, cost_s As cost, reverse_cost_s As reverse_cost, source, target FROM osm.ways',
        ARRAY(SELECT closest_node FROM fire_stations),
        60 * 1,
        directed := TRUE, 
        equicost := TRUE
    ) AS X INNER JOIN 
    osm.ways_vertices_pgr Y ON X.node = Y.id INNER JOIN
    osm.ways Z 
    ON Z.source = Y.id OR Z.target = Y.id;


-- OJ

-- Find nodes in catchment area
WITH D AS (
    SELECT X.from_v, F.address, X.seq, X.node, X.edge, Y.the_geom AS node_geom, X.cost/60.0 AS cost_in_minutes, X.agg_cost/60.0 AS agg_cost_in_minutes
    FROM 
        pgr_drivingDistance(
            'SELECT gid AS id, cost_s As cost, reverse_cost_s As reverse_cost, source, target FROM osm.ways',
            ARRAY(SELECT closest_node FROM fire_stations),
            60 * 1,
            directed := TRUE, 
            equicost := TRUE
        ) X INNER JOIN 
        osm.ways_vertices_pgr Y ON X.node = Y.id INNER JOIN 
        fire_stations F ON F.closest_node = X.from_v
    )
SELECT * FROM D;


-- Find edges that have both nodes accounted for in catchment area
WITH D AS (
    SELECT X.from_v, F.address, X.seq, X.node, X.edge, Y.the_geom AS node_geom, X.cost/60.0 AS cost_in_minutes, X.agg_cost/60.0 AS agg_cost_in_minutes
    FROM 
        pgr_drivingDistance(
            'SELECT gid AS id, cost_s As cost, reverse_cost_s As reverse_cost, source, target FROM osm.ways',
            ARRAY(SELECT closest_node FROM fire_stations),
            60 * 1,
            directed := TRUE, 
            equicost := TRUE
        ) AS X INNER JOIN 
        osm.ways_vertices_pgr Y ON X.node = Y.id INNER JOIN 
        fire_stations F ON F.closest_node = X.from_v
)
SELECT gid As id, name, the_geom As edge_geom
FROM osm.ways
WHERE source IN (SELECT node FROM D) AND target IN (SELECT node FROM D);



 



