﻿DROP TABLE IF EXISTS edges;

CREATE TABLE edges
(
    id integer PRIMARY KEY,
    edge_name varchar NOT NULL,
    the_geom geometry(LineString,2163) NOT NULL
);

ALTER TABLE edges ADD COLUMN source integer;
ALTER TABLE edges ADD COLUMN target integer;

TRUNCATE TABLE edges;

DROP TABLE IF EXISTs edges_vertices_pgr;

INSERT INTO edges (id, edge_name, the_geom)
VALUES
    (1,'SFO-DFW', ST_Transform(ST_GeomFromText('LINESTRING(-122.38 37.62, -97.04 32.90)',4326),2163)),
    (2,'DFW-BOS', ST_Transform(ST_GeomFromText('LINESTRING(-97.04 32.90, -71.01 42.36)',4326),2163)),
    (3,'BOS-SFO', ST_Transform(ST_GeomFromText('LINESTRING(-71.01 42.36, -122.38 37.62)',4326),2163));

SELECT id, edge_name, ST_AsText(the_geom), source, target FROM edges;

SELECT edge_name, the_geom FROM edges; -- OJ

SELECT pgr_createTopology('edges',1);

SELECT id, the_geom FROM edges_vertices_pgr; -- OJ

-----

TRUNCATE TABLE edges;

DROP TABLE IF EXISTS edges_vertices_pgr;

INSERT INTO edges (id, edge_name, the_geom)
VALUES
    (1,'SFO-DFW', ST_Transform(ST_GeomFromText('LINESTRING(-122.38 37.62, -97.04 32.90)',4326),2163)),
    (2,'DFW-BOS', ST_Transform(ST_GeomFromText('LINESTRING(-97.04 32.90, -71.01 42.36)',4326),2163)),
    (3,'BOS-SFO', ST_Transform(ST_GeomFromText('LINESTRING(-71.01 42.36, -122.38 37.62)',4326),2163)),
    (4,'SJC-DAL', ST_Transform(ST_GeomFromText('LINESTRING(-121.93 37.36, -96.85 32.85)',4326),2163));

SELECT pgr_createTopology('edges',100000, clean := TRUE);

SELECT id, edge_name, ST_AsText(the_geom), source, target FROM edges;

-----

TRUNCATE TABLE edges;

DROP TABLE IF EXISTS edges_vertices_pgr;

INSERT INTO edges (id, edge_name, the_geom)
VALUES
    (1,'SFO-DFW', ST_Transform(ST_GeomFromText('LINESTRING(-122.38 37.62, -97.04 32.90)',4326),2163)),
    (2,'DFW-BOS', ST_Transform(ST_GeomFromText('LINESTRING(-97.04 32.90, -71.01 42.36)',4326),2163)),
    (3,'BOS-CRP-INL-SFO', ST_Transform(ST_GeomFromText('LINESTRING(-71.01 42.36, -97.50 27.77, -93.40 48.57, -122.38 37.62)',4326),2163));
    
SELECT pgr_createTopology('edges',100000, clean := TRUE);

SELECT id, edge_name, ST_AsText(the_geom), source, target FROM edges;

-----

TRUNCATE TABLE edges;

DROP TABLE IF EXISTS edges_vertices_pgr;

INSERT INTO edges (id, edge_name, the_geom)
VALUES
    (1,'SFO-DFW', ST_Transform(ST_GeomFromText('LINESTRING(-122.38 37.62, -97.04 32.90)',4326),2163)),
    (2,'DFW-BOS', ST_Transform(ST_GeomFromText('LINESTRING(-97.04 32.90, -71.01 42.36)',4326),2163)),
    (3,'BOS-SFO', ST_Transform(ST_GeomFromText('LINESTRING(-71.01 42.36, -122.38 37.62)',4326),2163)),
    (4,'LAX-PHX', ST_Transform(ST_GeomFromText('LINESTRING(-118.41 33.94, -112.01 33.43)',4326),2163)),
    (5,'MIA-SEA', ST_Transform(ST_GeomFromText('LINESTRING(-80.29 25.80, -122.31 47.45)',4326),2163)),
    (6,'BOS-bos', ST_Transform(ST_GeomFromText('LINESTRING(-71.01 42.36, -71.01 42.36)',4326),2163));

SELECT pgr_createTopology('edges',1, clean := TRUE);

SELECT pgr_analyzeGraph('edges',1);

-----

SELECT ST_Transform(the_geom,2163) FROM tiger.state WHERE stusps NOT IN ('AK','AS','GU','HI','MP','PR','VI')

-----

DROP TABLE IF EXISTS edges;

CREATE TABLE edges
(
    id integer PRIMARY KEY,
    edge_name varchar NOT NULL,
    the_geom geometry(LineString,2163) NOT NULL,
    direction varchar NOT NULL
);

ALTER TABLE edges ADD COLUMN source integer;
ALTER TABLE edges ADD COLUMN target integer;

TRUNCATE TABLE edges;

DROP TABLE IF EXISTs edges_vertices_pgr;

INSERT INTO edges (id, edge_name, the_geom, direction)
VALUES
    (1,'SFO-DFW', ST_Transform(ST_GeomFromText('LINESTRING(-122.38 37.62, -97.04 32.90)',4326),2163), '>>>>'),
    (2,'DFW-BOS', ST_Transform(ST_GeomFromText('LINESTRING(-97.04 32.90, -71.01 42.36)',4326),2163), '>>>>'),
    (3,'BOS-SFO', ST_Transform(ST_GeomFromText('LINESTRING(-71.01 42.36, -122.38 37.62)',4326),2163), '<<<<');

SELECT pgr_createTopology('edges',1);

ALTER TABLE edges ADD COLUMN cost float;
ALTER TABLE edges ADD COLUMN reverse_cost float;

-- Type in Costs

SELECT cost || ' / ' || reverse_cost || ' ' || direction AS costs, the_geom FROM edges; -- OJ

-- Directed = TRUE by Default
 
SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost FROM edges', 1, 3, directed := TRUE);
SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost FROM edges', 1, 2, directed := FALSE);
SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost FROM edges', 2, 3);

SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost, reverse_cost FROM edges', 1, 3);
SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost, reverse_cost FROM edges', 1, 2);
SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost, reverse_cost FROM edges', 2, 3);

SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost, reverse_cost FROM edges', 1, 3, directed := FALSE);

SELECT * FROM pgr_dijkstra('SELECT id, source, target, reverse_cost FROM edges',1,2, directed := TRUE);
SELECT * FROM pgr_dijkstra('SELECT id, source, target, reverse_cost FROM edges',1,2, directed := FALSE);

SELECT * FROM pgr_dijkstra('SELECT id, source, target, cost, reverse_cost FROM edges',1,2, directed := FALSE);

SELECT id, ST_AsText(ST_Transform(the_geom,4326)) FROM edges_vertices_pgr;


