﻿SELECT * FROM mbta.mbta_arc_original;
SELECT * FROM mbta.mbta_node_original;

DROP TABLE IF EXISTS mbta.mbta_node;
CREATE TABLE mbta.mbta_node AS SELECT * FROM mbta.mbta_node_original WHERE NOT line LIKE '%SILVER%';

--OJ
SELECT line, geom FROM mbta.mbta_arc_original
UNION ALL
SELECT line, geom FROM mbta.mbta_node;

SELECT ST_NumGeometries(geom), ST_AsText(geom) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1;

-- OJ
SELECT 1 AS id, ST_GeometryN(geom,1) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1
UNION ALL
SELECT 2, ST_GeometryN(geom,2) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1;

SELECT 
    ST_Snap(
        (SELECT ST_GeometryN(geom,1) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
        (SELECT ST_GeometryN(geom,2) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
        1
    );

SELECT 
    ST_AsText(
        ST_Boundary(
            ST_Snap(
                (SELECT ST_GeometryN(geom,1) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
                (SELECT ST_GeometryN(geom,2) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
                1
            )
        )
    )
UNION ALL
SELECT ST_AsText(ST_Boundary(ST_GeometryN(geom,2))) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1;

-- OJ
WITH 
    X AS (
        SELECT 
            ST_Snap(
                (SELECT ST_GeometryN(geom,1) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
                (SELECT ST_GeometryN(geom,2) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
                1
            ) AS geom
    ),
    Y AS (
        SELECT ST_GeometryN(geom,2) AS geom FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1
    )        
SELECT ST_LineMerge(ST_Union(X.geom, Y.geom)) FROM X CROSS JOIN Y;

DROP TABLE IF EXISTS mbta.mbta_arc;

CREATE TABLE mbta.mbta_arc (LIKE mbta.mbta_arc_original);

ALTER TABLE mbta.mbta_arc ALTER COLUMN geom TYPE geometry(LINESTRING,26986);
INSERT INTO mbta.mbta_arc (gid, line, route, grade, shape_len, geom)
SELECT 
    gid,
    line, 
    route,
    grade, 
    shape_len,         
    ST_GeometryN(geom,1)
FROM mbta.mbta_arc_original
WHERE ST_NumGeometries(geom) = 1 AND NOT line LIKE '%SILVER%';

WITH Z AS (
    WITH 
        X AS (
            SELECT 
                ST_Snap(
                    (SELECT ST_GeometryN(geom,1) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
                    (SELECT ST_GeometryN(geom,2) FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1),
                    1
                ) AS geom
        ),
        Y AS (
            SELECT ST_GeometryN(geom,2) AS geom FROM mbta.mbta_arc_original WHERE ST_NumGeometries(geom) <> 1
        )        
    SELECT ST_LineMerge(ST_Union(X.geom, Y.geom)) AS geom FROM X CROSS JOIN Y
)
INSERT INTO mbta.mbta_arc (gid, line, route, shape_len, geom)
SELECT 
    X.gid, X.line, X.route, X.shape_len,
    Z.geom         
FROM mbta.mbta_arc_original X CROSS JOIN Z
WHERE ST_NumGeometries(X.geom) <> 1;


SELECT * FROM mbta.mbta_arc;

-- OJ
SELECT line, route, ST_LineMerge(ST_Union(geom)) AS geom
FROM mbta.mbta_arc 
GROUP BY line, route
ORDER BY line, route;

-- Output
SELECT line, route, ST_AsText(geom), ST_AsText(ST_Boundary(geom)) FROM (
    SELECT line, route, ST_LineMerge(ST_Union(geom)) AS geom
    FROM mbta.mbta_arc 
    GROUP BY line, route
) X;

-- Gap and overshoot near St Mary Station
UPDATE mbta.mbta_arc SET geom = ST_Snap(geom,ST_GeomFromText('POINT(232841 899752)', 26986), 5) WHERE line = 'GREEN' AND ST_Dwithin(geom, ST_GeomFromText('POINT(232841 899752)', 26986), 5);

-- Beg: Magic: Divide up by route, merge linestrings, resplit by station

DROP TABLE IF EXISTS mbta.mbta_lines;

CREATE TABLE mbta.mbta_lines AS 
WITH 
    X AS (
        SELECT line, COALESCE(track,route) AS route, geom
        FROM 
            mbta.mbta_arc
            LEFT JOIN 
            unnest(ARRAY['A', 'B', 'C', 'D', 'E', 'Ashmont', 'Braintree']) As track  
            ON 
                line = 'GREEN' AND left(route,7) LIKE ('%' || track || '%') OR 
                line = 'RED' AND length(track) > 5 AND route LIKE ('%' || track || '%')
    ),
    Y AS (
        SELECT line, route, (ST_Dump(ST_LineMerge(ST_Union(geom)))).geom As geom
        FROM X
        GROUP BY line, route
        ORDER BY line, route
    )
SELECT row_number() OVER() AS id, line, route, D.geom::geometry(LINESTRING,26986)
FROM 
    Y
    LEFT JOIN LATERAL 
    (
        SELECT 
            ST_Union(N.geom ORDER BY Y.geom <-> N.geom) AS geom 
        FROM mbta.mbta_node AS N 
        WHERE ST_DWithin(Y.geom, N.geom, 10) AND N.line ILIKE '%' || Y.line || '%'
    ) AS Z ON TRUE
    CROSS JOIN LATERAL 
    ST_Dump(COALESCE(ST_Split(ST_Snap(Y.geom, Z.geom, 10), Z.geom), Y.geom)) AS D;

-- End: Magic

-- Green C Line extra track beyond North Station needs to be removed
WITH X AS (
    SELECT L.id FROM mbta.mbta_lines AS L LEFT JOIN mbta.mbta_node AS N ON (ST_Intersects(ST_StartPoint(L.geom), N.geom)) WHERE N.gid IS NULL -- Vertices doesn't start at station
    UNION ALL
    SELECT L.id FROM mbta.mbta_lines AS L LEFT JOIN mbta.mbta_node AS N ON (ST_Intersects(ST_EndPoint(L.geom), N.geom)) WHERE N.gid IS NULL -- Vertices doesn't start at station
)
DELETE FROM mbta.mbta_lines WHERE id IN (SELECT id FROM X);

ALTER TABLE mbta.mbta_lines ADD COLUMN source integer;
ALTER TABLE mbta.mbta_lines ADD COLUMN target integer;

DROP TABLE IF EXISTS mbta.mbta_lines_vertices_pgr;

SELECT pgr_createTopology('mbta.mbta_lines', 1, 'geom', 'id', clean := TRUE);

SELECT * FROM mbta.mbta_lines_vertices_pgr;
SELECT * FROM mbta.mbta_node;

ALTER TABLE mbta.mbta_lines_vertices_pgr ADD COLUMN station varchar(40);
UPDATE mbta.mbta_lines_vertices_pgr X SET station = Y.station FROM mbta.mbta_node AS Y WHERE ST_Intersects(X.the_geom,Y.geom);