﻿SELECT P.id, P.name, P.geom, T.* 
FROM 
    pgr_eucledianTSP(
        $$SELECT id, ST_X(geom) AS x, ST_Y(geom) AS y FROM pois$$,
        1
    ) T INNER JOIN pois P ON T.node = P.id
ORDER BY T.seq;


--OJ TSP Euclidean path
SELECT P.id, P.name, T.*, COALESCE(ST_MakeLine(P.geom, LEAD(P.geom) OVER(ORDER BY T.seq)), P.geom) AS path_geom
FROM 
    pgr_eucledianTSP(
        $$
            SELECT 
                id, 
                ST_X(geom) AS x, 
                ST_Y(geom) AS y 
            FROM pois
        $$,
        1
    ) T INNER JOIN pois P ON T.node = P.id
ORDER BY T.seq;

DROP TABLE IF EXISTS tsp_cost_matrix;

SELECT 
    P1.id::bigint As start_vid, 
    P2.id::bigint As end_vid, 
    MAX(agg_cost) OVER (PARTITION BY least(P1.id,P2.id), greatest(P1.id,P2.id)) AS agg_cost 
INTO tsp_cost_matrix 
FROM 
    pgr_dijkstraCostMatrix(
        'SELECT gid AS id, source, target, cost_s AS cost, reverse_cost_s AS reverse_cost FROM osm.ways',
        (SELECT array_agg(DISTINCT closest_node_id) FROM pois)
    ) AS X INNER JOIN 
    pois P1 ON X.start_vid = P1.closest_node_id INNER JOIN 
    pois P2 ON X.end_vid = P2.closest_node_id
;


SELECT * FROM tsp_cost_matrix;


SELECT * 
FROM 
    pgr_TSP(
        'SELECT start_vid, end_vid, agg_cost FROM tsp_cost_matrix',
        1::bigint,
        1::bigint, 
        randomize := FALSE
    ) AS tsp
; -- 1 is the Seaport Hotel


SELECT T.seq, P.id, P.name
FROM 
    pgr_TSP(
        'SELECT start_vid, end_vid, agg_cost FROM tsp_cost_matrix', 
        1::bigint, 
        randomize := FALSE) T INNER JOIN 
    pois P ON T.node = P.id
ORDER BY T.seq;


-- OJ TSP with cost matrix (straight line)
SELECT T.seq, P.id, P.name, COALESCE(ST_MakeLine(P.geom, LEAD(P.geom) OVER(ORDER BY T.seq) ), P.geom) AS path_geom
FROM 
    pgr_TSP(
        'SELECT start_vid, end_vid, agg_cost FROM tsp_cost_matrix', 
        1::bigint, 
        randomize := FALSE
    ) T INNER JOIN 
    pois P ON T.node = P.id
ORDER BY T.seq;

-- OJ TSP with cost matrix (actual path)
SELECT R.path_id, ST_MakeLine(e.the_geom ORDER BY R.seq) AS  path_geom
FROM 
    pgr_dijkstraVia(
        $$SELECT gid AS id, source, target, cost_s As cost, reverse_cost_s As reverse_cost FROM osm.ways$$,
        ARRAY(SELECT P.closest_node_id FROM pois P INNER JOIN  pgr_TSP('SELECT start_vid, end_vid, agg_cost FROM tsp_cost_matrix', 1::bigint, randomize := FALSE) AS X ON P.id = X.node ORDER BY X.seq)
    ) R LEFT JOIN 
    osm.ways E ON R.edge = E.gid LEFT JOIN 
    pois P ON P.closest_node_id = R.node
GROUP BY R.path_id
ORDER BY R.path_id;